/**
 *  DWServerData.h
 *  Darkwood
 *
 *  Created by Mathieu LEDRU on 08/11/09.
 *
 *  BSD License:
 *  Copyright (c) 2009, Mathieu LEDRU
 *
 *  All rights reserved.
 *  Redistribution and use in source and binary forms, with or without modification,
 *  are permitted provided that the following conditions are met:
 *
 *  *   Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *  *   Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer
 *      in the documentation and/or other materials provided with the distribution.
 *  *   Neither the name of the Mathieu LEDRU nor the names of its contributors
 *      may be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 *  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 *  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, ServerData, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#import <Darkwood/DWApple.h>

typedef enum
{
	DWServerDataErrUrl,
	DWServerDataErrConnection,
	DWServerDataErrCancel,
} DWServerDataErr;

@protocol DWServerDataDelegate<NSObject>

- (void) serverDataProgress:(NSString *)aURLString progress:(float) aProgess; //[0,1] progression
- (void) serverDataFinish:(NSString *)aURLString data:(NSData*) aData;
- (void) serverDataFail:(NSString *)aURLString err:(DWServerDataErr) anErr;

@end

@class DWServerRequestQueue;
@class DWServerURLStringDelegate;

/**
 * DWServerData manage (shared) connections, cache disk management for URLStrings and delegation.
 * The cache stored is relative the DWServerData class name instance and context
 * Foreach URLString, we got the associated delegates, delegates dictionnary is maintained as we requestWithURLString with same URLString.
 */
@interface DWServerData : NSObject<DWServerDataDelegate> {
	NSString* context; //identifier that distinguish two DWServerData of the same type
	NSString* diskPath; //the location at which to store the on-disk cache.
	DWServerURLStringDelegate* delegates; //manage delegation request.
	DWServerRequestQueue* requestQueue; //let requestQueue manage the request queue list when requestWithURLString (shared among DWServerData instances).
}

-(id) initWithContext:(NSString*) aContext;

//request and single delegate
-(void) requestWithURLString:(NSString*) aURLString delegate:(id<DWServerDataDelegate>)aDelegate;

//request and multiple delegate
-(void) requestWithURLString:(NSString*) aURLString delegates:(id<DWServerDataDelegate>)theDelegates, ...;

- (void)cancelAllRequests;

/**
 * @return the data associated with aURLString, or nil if no data is associated with aURLString.
 */
- (NSData*) dataForURLString:(NSString*) aURLString;

/**
 * store aData associated with aURLString, unless aData is nil in which case the method instead attempts to delete data associated with aURLString.
 */
- (void) storeData:(NSData*) aData forURLString:(NSString*) aURLString;

@end
